@@ -3,7 +3,7 @@ use warnings;
 
 # ABSTRACT: load YAML from cache or disk, whichever seems better
 package YAML::CacheLoader;
-our $VERSION = '0.012';
+our $VERSION = '0.013';
 
 use base qw( Exporter );
 our @EXPORT_OK = qw( LoadFile DumpFile FlushCache FreshenCache);
@@ -35,16 +35,22 @@ sub LoadFile {
     my ($path, $force_reload) = @_;
 
     my $file_loc = path($path)->canonpath;    # realpath would be more accurate, but slower.
-
-    if (not $force_reload) {
-        my $from_cache = Cache::RedisDB->get(CACHE_NAMESPACE, $file_loc);
-        return $from_cache if $from_cache;    # Happy path
+    my $structure;
+    if ($force_reload) {
+        $structure = _load_and_cache($file_loc);
+    } else {
+        FreshenCache($file_loc);
+        $structure = Cache::RedisDB->get(CACHE_NAMESPACE, $file_loc) // _load_and_cache($file_loc);
     }
 
-    # Looks like we'll need to actually do some work, then.
-    my $structure = YAML::LoadFile($file_loc);    # Let this fail in whatever ways it might.
+    return $structure;
+}
 
-    Cache::RedisDB->set(CACHE_NAMESPACE, $file_loc, $structure, CACHE_SECONDS) if ($structure);
+sub _load_and_cache {
+    my $loc = shift;
+
+    my $structure = YAML::LoadFile($loc);    # Let this fail in whatever ways it might.
+    Cache::RedisDB->set(CACHE_NAMESPACE, $loc, $structure, CACHE_SECONDS) if ($structure);
 
     return $structure;
 }
@@ -89,32 +95,36 @@ sub FlushCache {
 FreshenCache();
 
 Freshen currently cached files which may be out of date, either by deleting the cache (for now deleted files) or reloading from the disk (for changed ones)
+
+May optionally provide a list of files to check, otherwise all known cached files are checked.
+
 Returns a stats hash-ref.
 
 =back
 =cut
 
 sub FreshenCache {
+    my (@file_list) = @_;
+
+    @file_list = _cached_files_list() unless @file_list;    # By default check all currently cached.
+
+    my @to_check = map { path($_) } @file_list;
     # A good rough cut is to see if something _might_ have changed in the meantime
     my $cutoff = time - CACHE_SECONDS;
 
-    my @cached_files = map { path($_) } _cached_files_list();
-
     my $stats = {
-        examined  => scalar @cached_files,
+        examined  => scalar @to_check,
         cleared   => 0,
         freshened => 0,
     };
 
-    foreach my $file (@cached_files) {
+    foreach my $file (@to_check) {
         if (!$file->exists) {
             $stats->{cleared}++ if (Cache::RedisDB->del(CACHE_NAMESPACE, $file->canonpath));    # Let's not cache things which don't exist.
         } elsif ((my $mtime = $file->stat->mtime) > $cutoff
             && (my $reloaded_ago = CACHE_SECONDS - Cache::RedisDB->ttl(CACHE_NAMESPACE, $file->canonpath)))
         {
-            # Now see if we might have reloaded the cache in the meantime.
-            my $reloaded = time - $reloaded_ago;
-            $stats->{freshened}++ if ($reloaded < $mtime && LoadFile($file, 1));
+            $stats->{freshened}++ if (time - $reloaded_ago < $mtime && LoadFile($file, 1));
         }
     }
 
